@@ -50,18 +50,35 @@ static int is_initials_word(const char * word)
 }
 
 /**
+ * Return TRUE if word is a proper name.
+ * XXX This is a cheap hack that works only in english, and is 
+ * broken for German!  We need to replace this with something
+ * language-specific. 
+ *
+ * Basically, if word starts with upper-case latter, we assume
+ * its a proper name, and that's that.
+ */
+static int is_proper_name(const char * word)
+{
+	return is_utf8_upper(word);
+}
+
+/**
+ * Returns true if the word can be interpreted as a number.
  * The ":" is included here so we allow "10:30" to be a number.
  * We also allow U+00A0 "no-break space"
  */
 static int is_number(const char * s)
 {
+	mbstate_t mbs;
 	int nb = 1;
 	wchar_t c;
 	if (!is_utf8_digit(s)) return FALSE;
 
+	memset(&mbs, 0, sizeof(mbs));
 	while ((*s != 0) && (0 < nb))
 	{
-		nb = mbtowc(&c, s, MB_CUR_MAX);
+		nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 		if (iswdigit(c)) { s += nb; }
 
 		/* U+00A0 no break space */
@@ -73,6 +90,25 @@ static int is_number(const char * s)
 	return TRUE;
 }
 
+/**
+ * Returns true if the word contains digits. 
+ */
+static int contains_digits(const char * s)
+{
+	mbstate_t mbs;
+	int nb = 1;
+	wchar_t c;
+
+	memset(&mbs, 0, sizeof(mbs));
+	while ((*s != 0) && (0 < nb))
+	{
+		nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+		if (iswdigit(c)) return TRUE;
+		s += nb;
+	}
+	return FALSE;
+}
+
 #if DONT_USE_REGEX_GUESSING
 /**
  * Returns TRUE iff it's an appropriately formed hyphenated word.
@@ -81,15 +117,17 @@ static int is_number(const char * s)
  */
 static int ishyphenated(const char * s)
 {
+	mbstate_t mbs;
 	wchar_t c;
 	int hyp, nonalpha;
 	hyp = nonalpha = 0;
 
 	if (*s == '-') return FALSE;
 
+	memset(&mbs, 0, sizeof(mbs));
 	while (*s != '\0')
 	{
-		int nb = mbtowc(&c, s, MB_CUR_MAX);
+		int nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 		if (0 > nb) break;
 
 		if (!iswalpha(c) && !iswdigit(c) && (*s!='.') && (*s!=',')
@@ -244,13 +282,15 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
 	char save[MB_LEN_MAX];
 	wchar_t c;
 	int nbl, nbh;
-	mbstate_t mbss;
+	mbstate_t mbs, mbss;
 
 	if (!is_utf8_upper(word)) return FALSE;
 
-	nbh = mbtowc (&c, word, MB_CUR_MAX);
+	memset(&mbs, 0, sizeof(mbs));
+	memset(&mbss, 0, sizeof(mbss));
+
+	nbh = mbrtowc (&c, word, MB_CUR_MAX, &mbs);
 	c = towlower(c);
-	nbl = wcrtomb(NULL, L'\0', &mbss);
 	nbl = wctomb_check(low, c, &mbss);
 	if (nbh != nbl)
 	{
@@ -300,7 +340,29 @@ static int separate_word(Sentence sent,
 
 	const char *r_stripped[MAX_STRIP];  /* these were stripped from the right */
 
-	/* Load affixes from the affix table.  */
+	/* First, see if we can already recognize the word as-is. If
+	 * so, then we are done. Else we'll try stripping prefixes, suffixes.
+	 */
+	strncpy(word, w, MIN(wend-w, MAX_WORD));
+	word[MIN(wend-w, MAX_WORD)] = '\0';
+	word_is_in_dict = FALSE;
+
+	if (is_proper_name(word))
+		word_is_in_dict = TRUE;
+	else if (boolean_dictionary_lookup(sent->dict, word))
+		word_is_in_dict = TRUE;
+	else if (is_initials_word(word))
+		word_is_in_dict = TRUE;
+	else if (is_first_word && downcase_is_in_dict (sent->dict,word))
+		word_is_in_dict = TRUE;
+
+	if (word_is_in_dict)
+	{
+		issue_sentence_word(sent, word);
+		return TRUE;
+	}
+
+	/* Set up affix tables.  */
 	if (sent->dict->affix_table != NULL)
 	{
 		Dictionary dict = sent->dict->affix_table;
@@ -317,6 +379,7 @@ static int separate_word(Sentence sent,
 		suffix = dict->suffix;
 	}
 
+	/* Strip off punctuation, etc. on teh left-hand side. */
 	for (;;)
 	{
 		for (i=0; i<l_strippable; i++)
@@ -368,49 +431,55 @@ static int separate_word(Sentence sent,
 		if (i == r_strippable) break;
 	}
 
-	/* Same as above, but with a twist: the only thing that can
-	 * preceed a units suffix is a number. This is so that we can
-	 * split up things like "12ft" (twelve feet) but not split up
-	 * things like "Delft blue". Multiple passes allow for
-	 * constructions such as 12sq.ft.
+	/* Is there a number in the word? If so, then search for
+	 * trailing units suffixes.
 	 */
-	n_r_stripped_save = n_r_stripped;
-	wend_save = wend;
-	for (; n_r_stripped < MAX_STRIP; n_r_stripped++) 
+	if (contains_digits(word))
 	{
-		size_t sz = MIN(wend-w, MAX_WORD);
-		strncpy(word, w, sz);
-		word[sz] = '\0';
-		if (wend == w) break;  /* it will work without this */
-
-		/* Number */
-		if (is_number(word))
+		/* Same as above, but with a twist: the only thing that can
+		 * preceed a units suffix is a number. This is so that we can
+		 * split up things like "12ft" (twelve feet) but not split up
+		 * things like "Delft blue". Multiple passes allow for
+		 * constructions such as 12sq.ft.
+		 */
+		n_r_stripped_save = n_r_stripped;
+		wend_save = wend;
+		for (; n_r_stripped < MAX_STRIP; n_r_stripped++) 
 		{
-			found_number = 1;
-			break;
-		}
+			size_t sz = MIN(wend-w, MAX_WORD);
+			strncpy(word, w, sz);
+			word[sz] = '\0';
+			if (wend == w) break;  /* it will work without this */
 
-		for (i=0; i < u_strippable; i++)
-		{
-			len = strlen(strip_units[i]);
-
-			/* the remaining w is too short for a possible match */
-			if ((wend-w) < len) continue;
-			if (strncmp(wend-len, strip_units[i], len) == 0)
+			/* Number */
+			if (is_number(word))
 			{
-				r_stripped[n_r_stripped] = strip_units[i];
-				wend -= len;
+				found_number = 1;
 				break;
 			}
-		}
-		if (i == u_strippable) break;
-	}
 
-	/* The root *must* be a number! */
-	if (0 == found_number)
-	{
-		wend = wend_save;
-		n_r_stripped = n_r_stripped_save;
+			for (i=0; i < u_strippable; i++)
+			{
+				len = strlen(strip_units[i]);
+
+				/* the remaining w is too short for a possible match */
+				if ((wend-w) < len) continue;
+				if (strncmp(wend-len, strip_units[i], len) == 0)
+				{
+					r_stripped[n_r_stripped] = strip_units[i];
+					wend -= len;
+					break;
+				}
+			}
+			if (i == u_strippable) break;
+		}
+
+		/* The root *must* be a number! */
+		if (0 == found_number)
+		{
+			wend = wend_save;
+			n_r_stripped = n_r_stripped_save;
+		}
 	}
 
 	/* Now we strip off suffixes...w points to the remaining word, 
@@ -421,7 +490,12 @@ static int separate_word(Sentence sent,
 	word[MIN(wend-w, MAX_WORD)] = '\0';
 	word_is_in_dict = FALSE;
 
-	if (boolean_dictionary_lookup(sent->dict, word))
+	/* If the first letter is upper-case, then assume its
+	 * a proper name, and so we are done.
+	 */
+	if (is_proper_name(word))
+		word_is_in_dict = TRUE;
+	else if (boolean_dictionary_lookup(sent->dict, word))
 		word_is_in_dict = TRUE;
 	else if (is_initials_word(word))
 		word_is_in_dict = TRUE;
@@ -440,7 +514,7 @@ static int separate_word(Sentence sent,
 			{
 				len = strlen(suffix[i]);
 
-				/* the remaining w is too short for a possible match */
+				/* The remaining w is too short for a possible match */
 				if ((wend-w) < len) continue;
 				if (strncmp(wend-len, suffix[i], len) == 0) s_ok=1;
 			}
@@ -499,7 +573,7 @@ static int separate_word(Sentence sent,
 			}
 		}
 	}
-	
+
 	/* word is now what remains after all the stripping has been done */
 
 	if (n_r_stripped >= MAX_STRIP)
@@ -576,6 +650,7 @@ int separate_sentence(const char * s, Sentence sent)
 	const char *t;
 	int i, is_first, quote_found;
 	Dictionary dict = sent->dict;
+	mbstate_t mbs;
 
 	for(i=0; i<MAX_SENTENCE; i++) sent->post_quote[i] = 0;
 	sent->length = 0;
@@ -583,16 +658,14 @@ int separate_sentence(const char * s, Sentence sent)
 	if (dict->left_wall_defined)
 		if (!issue_sentence_word(sent, LEFT_WALL_WORD)) return FALSE;
 
-#ifndef __CYGWIN__
 	/* Reset the multibyte shift state to the initial state */
-	i = mbtowc(NULL, NULL, MB_CUR_MAX);
-#endif
+	memset(&mbs, 0, sizeof(mbs));
 
 	is_first = TRUE;
 	for(;;) 
 	{
 		wchar_t c;
-		int nb = mbtowc(&c, s, MB_CUR_MAX);
+		int nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 		quote_found = FALSE;
 
 		if (0 > nb) goto failure;
@@ -602,19 +675,19 @@ int separate_sentence(const char * s, Sentence sent)
 		{
 			s += nb;
 			if (*s == '\"') quote_found = TRUE;
-			nb = mbtowc(&c, s, MB_CUR_MAX);
+			nb = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
 			if (0 > nb) goto failure;
 		}
 
 		if (*s == '\0') break;
 
 		t = s;
-		nb = mbtowc(&c, t, MB_CUR_MAX);
+		nb = mbrtowc(&c, t, MB_CUR_MAX, &mbs);
 		if (0 > nb) goto failure;
 		while (!iswspace(c) && (c != '\"') && (c != 0) && (nb != 0))
 		{
 			t += nb;
-			nb = mbtowc(&c, t, MB_CUR_MAX);
+			nb = mbrtowc(&c, t, MB_CUR_MAX, &mbs);
 			if (0 > nb) goto failure;
 		}
 
